
const St = imports.gi.St;
const Shell = imports.gi.Shell;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;
const AltTab = imports.ui.altTab;
const Dash = imports.ui.dash;
const Lang = imports.lang;
const Gdk = imports.gi.Gdk;
const GnomeDesktop = imports.gi.GnomeDesktop;
const LD = imports.gi.LunarDate;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

let replacementFunc = [], ld = new LD.Date, settingsChanged = [], localSettings = {};
let settings;

ld.setDate = function (date) {
  this.set_solar_date(date.getFullYear(), (1+date.getMonth()), date.getDate(), date.getHours());
};

ld.setDateNoon = function (date) {
  this.set_solar_date(date.getFullYear(), (1+date.getMonth()), date.getDate(), 12);
};

let decode_utf8 = "﻿" == "\ufeff" ? function (i) { return i }
	: function (i) {
  let a, b, c, d;
  let o = "", j = a = b = c = d = 0, f = String.fromCharCode;

  while (j < i.length) {
    a = i.charCodeAt(j);
    if (a <= 191) {
      o += f(a);
      j++;
    } else if (a <= 223) {
      b = i.charCodeAt(j + 1);
      o += f(((a & 31) << 6) | (b & 63));
      j += 2;
    } else if (a <= 239) {
      b = i.charCodeAt(j + 1);
      c = i.charCodeAt(j + 2);
      o += f(((a & 15) << 12) | ((b & 63) << 6) | (c & 63));
      j += 3;
    } else {
      b = i.charCodeAt(j + 1);
      c = i.charCodeAt(j + 2);
      d = i.charCodeAt(j + 3);
      a = (((a & 7) << 18) | ((b & 63) << 12) | ((c & 63) << 6) | (d & 63)) - 0x10000;
      o += f(0xD800 + (a >> 10), 0xDC00 + (a & 0x3FF));
      j += 4;
    }
  }

  return o;
};

function init() {
}

function _make_new_with_args (my_class, args) {
  return new (Function.prototype.bind.apply(my_class,
	[null].concat(Array.prototype.slice.call(args))))();
}

function enable() {
  settings = Convenience.getSettings();
  settings.connect('changed', function() {
  	  for (x in settingsChanged) {
  		settingsChanged[x]();
  	  }
  	});

  let dm = Main.panel.statusArea.dateMenu;

  let cal = dm._calendar;
  let ev = dm._eventList;
  let cny_now;

  replacementFunc.calendarUpdater = cal._update;
  replacementFunc.originalMonthHeader = cal._headerFormat;
  replacementFunc.evAddPeriod = ev._addPeriod;

  let fixupHeader = _("%A %B %e, %Y").match(/%Y[^%]+%/);
  if (fixupHeader) {
	cal._headerFormat = cal._headerFormat.replace(/%Y.%/, fixupHeader);
  }

  dm._clock.run_dispose();
  dm._clock = new GnomeDesktop.WallClock();

  let replacementUpdateClockAndDate = function () {
	dm._updateClockAndDate.apply(dm, arguments);
	ld.setDate(new Date());
	cny_now = ld.strftime("%(shengxiao)");
	dm._date.text = dm._date.text + (dm._date.text.match(/[.,]/) ? ", " : "\u2001") +
	cny_now + "\n" + ld.strftime(decode_utf8("%(NIAN)年%(YUE)月%(RI)日"));

	let show_date = settings.get_boolean('show-date');
	dm._clockDisplay.text = dm._clockDisplay.text +
	(show_date ? "\u2001" + ld.strftime(decode_utf8("%(YUE)月%(RI)日")) : "") +
	(settings.get_boolean('show-time') ? ld.strftime(show_date ? decode_utf8("%(SHI)时") : "%(SHI)") : "");
  }

  dm._clock.connect('notify::clock', Lang.bind(dm, replacementUpdateClockAndDate));
  replacementUpdateClockAndDate();

  let _toHex = function (number) {
	let t = (number * 65535).toString(16);
	while (t.length < 4) t = '0' + t;
	return t;
  };

  settingsChanged.updateLocal = function () {
	let color = new Gdk.RGBA;
	color.parse('white');
	color.parse(settings.get_string('jieri-color'));
	localSettings.jieri_color = "#" + _toHex(color.red) + _toHex(color.green) + _toHex(color.blue);
  };
  settingsChanged.updateLocal();

  settingsChanged.refreshClock = replacementUpdateClockAndDate;

  cal._update = function () {
	let orig_button = St.Button;
	let orig_date = Date;
	let iter_date = new orig_date();

	Date = function () {
	  let new_date = _make_new_with_args(orig_date, arguments);
	  if (arguments.length > 0 && arguments[0] instanceof orig_date) {
		iter_date = new_date;
	  }
	  return new_date;
	};

	St.Button = function () {
	  let fat_button = false;
	  if (arguments[0].label == iter_date.getDate().toString()) {
		ld.setDateNoon(iter_date);
		let yd = settings.get_boolean('show-calendar') ? ld.strftime("%(ri)") : "";
		let jr = settings.get_boolean('jieri') ? ld.strftime("%(jieri)") : "";
		fat_button = yd != "";
		arguments[0].label = (jr != "" ? "<span weight='bold' color='" + localSettings.jieri_color + "'>" : "") +
		(yd != "" ? "<u> " : "") + arguments[0].label + (yd != "" ? " </u>" : "") +
		(jr != "" ? "</span>" : "") + (yd != "" ? "\n<small>" + 
		 (yd == "1" ? ld.strftime(decode_utf8("%(YUE)月")) : ld.strftime("%(RI)")) + "</small>" : "");
	  }
	  let new_button = _make_new_with_args(orig_button, arguments);
	  if (fat_button) {
		new_button.set_width(cal._digitWidth * 5);
	  }
	  return new_button;
	};

	replacementFunc.calendarUpdater.apply(this, arguments);

	ld.setDateNoon(this._selectedDate);
	let cny = ld.strftime("%(shengxiao)");
	if (cny != cny_now) {
	  this._monthLabel.text = this._monthLabel.text + " / " + cny;
	}

	St.Button = orig_button;
	Date = orig_date;
  };

  ev._addPeriod = function () {
	let one_day = arguments[1].getDate() == arguments[2].getDate();
	let tomorrow = arguments[0] == _("Tomorrow");
	let named_day = tomorrow || arguments[0] == _("Today");

	if (one_day) {
	  ld.setDateNoon(arguments[1]);

	  if (!named_day) {
		let cny = ld.strftime("%(shengxiao)");
		arguments[0] = arguments[0] + "\u2001" +
		  ld.strftime(decode_utf8((cny != cny_now ? "%(NIAN)年" : "") + "%(YUE)月%(RI)日"));
	  }
	}

	let orig_box = St.BoxLayout;
	let capturedBoxes = [];
	St.BoxLayout = function () {
	  let new_box = _make_new_with_args(orig_box, arguments);
	  if (arguments.length && arguments[0].style_class) {
		capturedBoxes[arguments[0].style_class] = new_box;
	  }
	  return new_box;
	};

	replacementFunc.evAddPeriod.apply(this, arguments);

	St.BoxLayout = orig_box;

	if (one_day) {
	  if (!tomorrow) {
		if (settings.get_boolean('gen-zhi')) {
		  this._addEvent(null, capturedBoxes['events-time-box'], capturedBoxes['events-event-box'], false, null, decode_utf8("干支"), ld.strftime(decode_utf8("%(Y60)年%(M60)月%(D60)日")));
		}
		if (settings.get_boolean('ba-zi')) {
		  this._addEvent(null, capturedBoxes['events-time-box'], capturedBoxes['events-event-box'], false, null, decode_utf8("八字"), ld.strftime(decode_utf8("%(Y8)年%(M8)月%(D8)日")));
		}
	  }

	  let jr = settings.get_boolean('jieri') ? ld.strftime("%(jieri)") : "";
	  if (jr != "") {
		this._addEvent(null, capturedBoxes['events-time-box'], capturedBoxes['events-event-box'], false, null, decode_utf8("节日"), ld.get_jieri("\n"));
	  }
	}
  };
}

function disable() {
  let dm = Main.panel.statusArea.dateMenu;

  dm._calendar._update = replacementFunc.calendarUpdater;
  dm._calendar._headerFormat = replacementFunc.originalMonthHeader;
  dm._eventList._addPeriod = replacementFunc.evAddPeriod;

  dm._clock.run_dispose();
  dm._clock = new GnomeDesktop.WallClock();
  dm._clock.connect('notify::clock', Lang.bind(dm, dm._updateClockAndDate));
  dm._updateClockAndDate();

  settingsChanged = [];
  settings.run_dispose();
  settings = null;

  delete replacementFunc.calendarUpdater;
  delete replacementFunc.originalMonthHeader;
  delete replacementFunc.evAddPeriod;
}
